#ifndef POKEPLATINUM_UNK_0206CCB0_H
#define POKEPLATINUM_UNK_0206CCB0_H

#include "struct_decls/struct_0200B358_decl.h"
#include "struct_decls/struct_0202440C_decl.h"
#include "struct_decls/struct_02025E6C_decl.h"
#include "struct_decls/struct_0203CDB0_decl.h"
#include "struct_decls/struct_0206D140_decl.h"
#include "struct_defs/pokemon.h"
#include "struct_decls/struct_021C0794_decl.h"
#include "overlay006/struct_ov6_022465F4_decl.h"

int sub_0206CD00(int param0, UnkStruct_0203CDB0 * param1, UnkStruct_0200B358 * param2, UnkStruct_ov6_022465F4 * param3, u16 * param4);
BOOL sub_0206CD2C(int param0, UnkStruct_0203CDB0 * param1, UnkStruct_ov6_022465F4 * param2);
void sub_0206CF14(UnkStruct_0202440C * param0, Pokemon * param1, int param2, int param3, int param4);
void sub_0206CF48(UnkStruct_0202440C * param0, Pokemon * param1, int param2);
void sub_0206CF9C(UnkStruct_0202440C * param0, int param1);
void sub_0206CFB4(UnkStruct_0202440C * param0, int param1);
void sub_0206CFCC(UnkStruct_0202440C * param0, int param1);
void sub_0206CFE4(UnkStruct_0202440C * param0, BOOL param1, u16 param2);
void sub_0206D000(UnkStruct_0202440C * param0);
void sub_0206D018(UnkStruct_0202440C * param0, Pokemon * param1);
void sub_0206D048(UnkStruct_0202440C * param0, Pokemon * param1);
void sub_0206D088(UnkStruct_0202440C * param0, u8 param1, const UnkStruct_02025E6C * param2);
void sub_0206D0C8(UnkStruct_0202440C * param0, u16 param1);
void sub_0206D0F0(UnkStruct_0202440C * param0);
void sub_0206D104(UnkStruct_0202440C * param0);
void sub_0206D12C(UnkStruct_0202440C * param0);
UnkStruct_0206D140 * sub_0206D140(int param0);
void sub_0206D158(UnkStruct_0206D140 * param0);
void sub_0206D160(UnkStruct_0206D140 * param0, Pokemon * param1, int param2, int param3, u32 param4);
void sub_0206D1B8(UnkStruct_0203CDB0 * param0, const UnkStruct_0206D140 * param1, int param2);
void sub_0206D340(UnkStruct_0203CDB0 * param0, BOOL param1, u16 param2, Pokemon * param3);
void sub_0206D424(UnkStruct_0203CDB0 * param0);
void sub_0206D430(UnkStruct_0203CDB0 * param0);
void sub_0206D4AC(UnkStruct_0203CDB0 * param0, u16 param1);
void sub_0206D504(UnkStruct_021C0794 * param0, u16 param1, u8 param2);
void sub_0206D578(UnkStruct_0203CDB0 * param0, Pokemon * param1);
void sub_0206D60C(UnkStruct_0203CDB0 * param0, Pokemon * param1);
void sub_0206D6C8(UnkStruct_0203CDB0 * param0, int param1, int param2);
void sub_0206D720(UnkStruct_0203CDB0 * param0);
void sub_0206D7C4(UnkStruct_0203CDB0 * param0);
void sub_0206D90C(UnkStruct_0202440C * param0, Pokemon * param1, u16 param2);
void sub_0206D914(UnkStruct_0203CDB0 * param0, u16 param1, u8 param2, u16 param3);
void sub_0206D9B4(UnkStruct_0202440C * param0, Pokemon * param1, u8 param2);
void sub_0206DAB8(UnkStruct_0203CDB0 * param0, const UnkStruct_02025E6C * param1);
void sub_0206DAD4(UnkStruct_0203CDB0 * param0, const UnkStruct_02025E6C * param1);
void sub_0206DB20(UnkStruct_0203CDB0 * param0);
void sub_0206DB5C(UnkStruct_0203CDB0 * param0, u8 param1);
void sub_0206DBB0(UnkStruct_021C0794 * param0, u32 param1, Pokemon * param2, BOOL param3);
void sub_0206DC6C(UnkStruct_0203CDB0 * param0, u32 param1, Pokemon * param2);
void sub_0206DD38(UnkStruct_0203CDB0 * param0, u32 param1, u32 param2, u32 param3);
void sub_0206DDB8(UnkStruct_021C0794 * param0, Pokemon * param1, u32 param2);
void sub_0206DEEC(UnkStruct_0203CDB0 * param0, u16 param1, u16 param2);
void sub_0206DF60(UnkStruct_0203CDB0 * param0, u16 param1);
void sub_0206DFE0(UnkStruct_021C0794 * param0);
void sub_0206E060(UnkStruct_021C0794 * param0);
void sub_0206E0E0(UnkStruct_0203CDB0 * param0, u16 param1);
void sub_0206E174(UnkStruct_0203CDB0 * param0, u16 param1);
void sub_0206E264(UnkStruct_0203CDB0 * param0, u16 param1);
void sub_0206E2BC(UnkStruct_0203CDB0 * param0, u16 param1);
void sub_0206E398(UnkStruct_0203CDB0 * param0, u16 param1);
void sub_0206E414(UnkStruct_0203CDB0 * param0, u16 param1);
void sub_0206E448(UnkStruct_0203CDB0 * param0, u16 param1);
void sub_0206E4DC(UnkStruct_0203CDB0 * param0, u16 param1);
void sub_0206E5A0(UnkStruct_0203CDB0 * param0, u16 param1);
void sub_0206E668(UnkStruct_0203CDB0 * param0, u16 param1);
void sub_0206E6A8(UnkStruct_0203CDB0 * param0, u16 param1);
void sub_0206E6E8(UnkStruct_0203CDB0 * param0, u16 param1);
void sub_0206E728(UnkStruct_0203CDB0 * param0, u16 param1);
void sub_0206E768(UnkStruct_0203CDB0 * param0, u16 param1);
void sub_0206F2F0(UnkStruct_021C0794 * param0);

#endif // POKEPLATINUM_UNK_0206CCB0_H
